@@ -1,9 +1,9 @@
 import platform
 
-# check that on Windows
 if platform.system() != 'Windows':
-	raise RuntimeError( 'Invalid Operating System: easy_biologic can only be used on Windows.' )
+	raise OSError( 'easy_biologic can only be used on Windows.' )
 
+from ._version import __version__, __version_info__
 
 from .device import BiologicDevice
 from .program import BiologicProgram
