@@ -44,21 +44,21 @@ def parse( data, info, fields = None, device = None ):
     :param device: BioLogic device. Necessary if fields are not defined.
     :returns: A list of namedtuples representing the data.
     """
+    if ( fields is None ) and ( device is None ):
+        raise ValueError( 'Both fields and device are None, at least one must be defined.' )
+
     rows = info.NbRows
     cols = info.NbCols
     technique = ecl.TechniqueId( info.TechniqueID )
 
-    if fields is None and device is not None:
+    if fields is None:
+        # get fields from device
         fields = (
-            SP300_Fields[ techinque ]
-            if device.kind is ecl.DeviceCodes.KBIO_DEV_SP300
-            else VMP3_Fields[ technique ]
+            SP300_Fields[ technique ]
+            if ecl.is_in_SP300_family( device.kind ) else
+            VMP3_Fields[ technique ]
         )
 
-    if fields is None and device is None:
-        raise ValueError( 'Both fields and device not defined.' )
-
-
     if isinstance( fields, tuple ):
         fields = fields[ info.ProcessIndex ]
 
@@ -79,7 +79,8 @@ def parse( data, info, fields = None, device = None ):
 
     # group data
     parsed = [
-        Datum( *data[ i : i + cols ] ) for i in range( 0, rows* cols, cols )
+        Datum( *data[ i : i + cols ] )
+        for i in range( 0, rows* cols, cols )
     ]
 
     return parsed
@@ -174,6 +175,32 @@ class VMP3_Fields():
     )
 
 
+    GEIS = (
+        [  # process == 0
+            FI( 't_high',  INT32  ),
+            FI( 't_low',   INT32  ),
+            FI( 'voltage', SINGLE ),
+            FI( 'current', SINGLE )
+        ],
+        [  # process == 1
+            FI( 'frequency',           SINGLE ),
+            FI( 'abs_voltage',         SINGLE ),
+            FI( 'abs_current',         SINGLE ),
+            FI( 'impendance_phase',    SINGLE ),
+            FI( 'voltage',             SINGLE ),
+            FI( 'current',             SINGLE ),
+            FI( 'empty1',              INT32  ),
+            FI( 'abs_voltage_ce',      SINGLE ),
+            FI( 'abs_current_ce',      SINGLE ),
+            FI( 'impendance_ce_phase', SINGLE ),
+            FI( 'voltage_ce',          SINGLE ),
+            FI( 'empty2',              INT32  ),
+            FI( 'empty3',              INT32  ),
+            FI( 'time',                SINGLE )
+        ]
+    )    
+
+
 class SP300_Fields():
     """
     Holds technique field definitions.
@@ -235,3 +262,28 @@ class SP300_Fields():
             FI( 'time',                SINGLE )
         ]
     )
+
+    GEIS = (
+        [  # process == 0
+            FI( 't_high',  INT32  ),
+            FI( 't_low',   INT32  ),
+            FI( 'voltage', SINGLE ),
+            FI( 'current', SINGLE )
+        ],
+        [  # process == 1
+            FI( 'frequency',           SINGLE ),
+            FI( 'abs_voltage',         SINGLE ),
+            FI( 'abs_current',         SINGLE ),
+            FI( 'impendance_phase',    SINGLE ),
+            FI( 'voltage',             SINGLE ),
+            FI( 'current',             SINGLE ),
+            FI( 'empty1',              INT32  ),
+            FI( 'abs_voltage_ce',      SINGLE ),
+            FI( 'abs_current_ce',      SINGLE ),
+            FI( 'impendance_ce_phase', SINGLE ),
+            FI( 'voltage_ce',          SINGLE ),
+            FI( 'empty2',              INT32  ),
+            FI( 'empty3',              INT32  ),
+            FI( 'time',                SINGLE )
+        ]
+    )
