@@ -1,8 +1,7 @@
 #!/usr/bin/env python
 # coding: utf-8
 
-# # EC Errors
-
+# EC Errors
 
 class EcError( Exception ):
     """
@@ -26,6 +25,11 @@ class EcError( Exception ):
             'Function failed.'
         ),
 
+        -7: (
+            'ERR_GEN_NOCHANNELSELECTED',
+            'No channel selected'
+        ),
+
         -9: (
             'ERR_GEN_ECLAB_LOADED',
             'ECLab firmware laoded on instrument.'
@@ -41,6 +45,11 @@ class EcError( Exception ):
             'Function of the library already in progress.'
         ),
 
+        -102: (
+            'ERR_INSTR_TOOMANYDATA',
+            'Too many data to transfer from the instrument (device error)'
+        ),
+
         -200: (
             'ERR_COMM_COMMFAILED',
             'Communication with the instrument failed.'
@@ -104,13 +113,9 @@ class EcError( Exception ):
                 ( code, message ) = EcError.errors[ value ]
 
             except KeyError as err:
-                raise ValueError( 'Unknown error value {}.'.format( value ) )
+                raise ValueError( f'Unknown error value {value}.' )
 
-            out = '{code} ({value}): {message}'.format(
-                value = value,
-                code = code,
-                message = message
-            )
+            out = f'{code} ({value}): {message}'
 
         else:
             # no error value
