@@ -21,7 +21,7 @@
 #
 # **get_hardware_configuration( idn, ch ):** Returns a HarwareConf struct of the given device channel.
 #
-# **set_hardware_configuration( idn, ch ):** Sets the hardware configuration of the given device channel.
+# **set_hardware_configuration( idn, ch, mode, connection ):** Sets the hardware configuration of the given device channel.
 #
 # **load_technique( idn, ch, technique, params, first = True, last = True, verbose = False ):**
 # Loads the technique with parameter on the given device channel.
@@ -46,6 +46,8 @@
 #
 # **raise_exception( err ):** Raises an exception based on a calls error code.
 #
+# **is_in_SP300_family( device_code ):** Determines if the given device is in the SP300 device family.
+#
 # #### Enum Classes
 # **DeviceCodes:** Device code for identifying model. <br>
 # Values: [ KBIO_DEV_VMP, KBIO_DEV_VMP2, KBIO_DEV_MPG, KBIO_DEV_BISTAT, KBIO_DEV_MCS_200, KBIO_DEV_VMP3, KBIO_DEV_VSP, KBIO_DEV_HCP803, KBIO_DEV_EPP400, KBIO_DEV_EPP4000, KBIO_DEV_BISTAT2, KBIO_DEV_FCT150S, KBIO_DEV_VMP300, KBIO_DEV_SP50, KBIO_DEV_SP150, KBIO_DEV_FCT50S, KBIO_DEV_SP300, KBIO_DEV_CLB500, KBIO_DEV_HCP1005, KBIO_DEV_CLB2000, KBIO_DEV_VSP300, KBIO_DEV_SP200, KBIO_DEV_MPG2, KBIO_DEV_ND1, KBIO_DEV_ND2, KBIO_DEV_ND3, KBIO_DEV_ND4, KBIO_DEV_SP240, KBIO_DEV_MPG205, KBIO_DEV_MPG210, KBIO_DEV_MPG220, KBIO_DEV_MPG240, KBIO_DEV_UNKNOWN ]
@@ -66,7 +68,7 @@
 # Values: [ GROUNDED, FLOATING ]
 #
 # **TechniqueId:** ID of the technique. (Not fully implemented.) <br>
-# Values: [ NONE, OCV, CA, CP, CV, PEIS, CALIMIT ]
+# Values: [ NONE, OCV, CA, CP, CV, PEIS, GEIS, CALIMIT ]
 #
 # **ChannelState:** State of the channel. <br>
 # Values: [ STOP, RUN, PAUSE ]
@@ -96,6 +98,11 @@
 #
 # **DataInfo:** Metadata of measured data. <br>
 # Fields: [ IRQskipped, NbRows, NbCols, TechniqueIndex, TechniqueID, processIndex, loop, StartTime, MuxPad ]
+#
+# #### Constants
+# **VMP3_DEVICE_FAMILY:** Set of DeviceCodes in the VMP3 device family.
+#
+# **SP300_DEVICE_FAMILY:** Set of DeviceCodes in the SP300 device family.
 
 
 # standard imports
@@ -104,14 +111,14 @@ import os
 import asyncio
 import ctypes as c
 import platform
-import pkg_resources
+# import pkg_resources
 from enum import Enum
 
 from .ec_errors import EcError
 from .. import common
 
-# ## Constants
 
+# Constants
 
 class DeviceCodes( Enum ):
     """
@@ -205,7 +212,7 @@ class IRange( Enum ):
     m1   = 7
     m10  = 8
     m100 = 9
-    a1   = 10 # 1 amp
+    a1   = 10  # 1 amp
 
     KEEP    = -1
     BOOSTER = 11
@@ -219,7 +226,6 @@ class ERange( Enum ):
     v2_5 = 0
     v5   = 1
     v10  = 2
-
     AUTO = 3
 
 
@@ -243,6 +249,7 @@ class TechniqueId( Enum ):
     CP    = 102 # chrono-potentiometry
     CV    = 103 # cyclic voltammetry
     PEIS  = 104 # potentio electrochemical impedance
+    GEIS  = 107 # galvano electrochemical impedance
 
     CALIMIT = 157 # chrono-amperometry with limits
 
@@ -394,8 +401,53 @@ class DataInfo( c.Structure ):
     ]
 
 
-# ## DLL Methods
+# device families
 
+# TODO [2]: Incorporate unknown device codes
+# Should also include
+# VMP3E, VSP3E, SP50E, SP150E
+# but do not have their device codes.
+VMP3_DEVICE_FAMILY = {
+    DeviceCodes.KBIO_DEV_VMP2,
+    DeviceCodes.KBIO_DEV_VMP3,
+    DeviceCodes.KBIO_DEV_BISTAT,
+    DeviceCodes.KBIO_DEV_BISTAT2,
+    DeviceCodes.KBIO_DEV_MCS_200,
+    DeviceCodes.KBIO_DEV_VSP,
+    DeviceCodes.KBIO_DEV_SP50,
+    DeviceCodes.KBIO_DEV_SP150,
+    DeviceCodes.KBIO_DEV_FCT50S,
+    DeviceCodes.KBIO_DEV_FCT150S,
+    DeviceCodes.KBIO_DEV_CLB500,
+    DeviceCodes.KBIO_DEV_CLB2000,
+    DeviceCodes.KBIO_DEV_HCP803,
+    DeviceCodes.KBIO_DEV_HCP1005,
+    DeviceCodes.KBIO_DEV_MPG2,
+    DeviceCodes.KBIO_DEV_MPG205,
+    DeviceCodes.KBIO_DEV_MPG210,
+    DeviceCodes.KBIO_DEV_MPG220,
+    DeviceCodes.KBIO_DEV_MPG240
+}
+
+    
+# TODO [2]: Incorporate unknown device codes
+# Should also include
+# SP100, BP300
+# but do not have their device codes.
+SP300_DEVICE_FAMILY = {
+    DeviceCodes.KBIO_DEV_SP200,
+    DeviceCodes.KBIO_DEV_SP300,
+    DeviceCodes.KBIO_DEV_VSP300,
+    DeviceCodes.KBIO_DEV_VMP300,
+    DeviceCodes.KBIO_DEV_SP240
+}
+
+# TODO [2]: Incorporate unknown device codes.
+# Do not know which family these belong to.
+# KBIO_DEV_VMP, KBIO_DEV_MPG, KBIO_DEV_EPP400, KBIO_DEV_EPP4000
+
+
+# DLL Methods
 
 #--- init ---
 
@@ -798,17 +850,17 @@ def init_channels( idn, chs, force_reload = False, bin_file = None, xlx_file = N
         )
     )
 
-    logging.debug( '[easy-biologic] Initializing channels {} on device {}.'.format( chs, idn.value ) )
+    logging.debug( f'[easy-biologic] Initializing channels {chs} on device {idn.value}.' )
     err = BL_LoadFirmware(
-            idn,
-            c.byref( active ),
-            c.byref( results ),
-            length,
-            show_gauge,
-            force_reload,
-            bin_file,
-            xlx_file
-        )
+        idn,
+        c.byref( active ),
+        c.byref( results ),
+        length,
+        show_gauge,
+        force_reload,
+        bin_file,
+        xlx_file
+    )
 
     validate( err )
 
@@ -1544,8 +1596,7 @@ async def get_data_async( idn, ch ):
     return ( data, info, values )
 
 
-# ## Helper Functions
-
+# Helper Functions
 
 def validate( err ):
     """
@@ -1595,10 +1646,10 @@ def technique_file( technique, device = None ):
     """
     file_type = '.ecc'
     sp300_mod = '4'
-
+    
     if (
         device is not None and
-        device is DeviceCodes.KBIO_DEV_SP300 and
+        is_in_SP300_family(device) and
         not technique.endswith( sp300_mod )
     ):
         # modify technqiues for SP-300 devices
@@ -1607,5 +1658,13 @@ def technique_file( technique, device = None ):
     if not technique.endswith( file_type ):
         # append file type extenstion if needed
         technique += file_type
-
+    
     return technique.lower()
+
+
+def is_in_SP300_family( device_code ):
+    """
+    :param device_code: DeviceCode.
+    :returns: Whether the device is in the SP300 (VMP_300) family or not.
+    """
+    return ( device_code in SP300_DEVICE_FAMILY )
