@@ -370,7 +370,6 @@ class BiologicProgram( ABC ):
                     write_header = self._write_header
                 )
 
-
         except Exception as err:
             self._writes_failed += 1
 
@@ -510,7 +509,8 @@ class BiologicProgram( ABC ):
             parsed = dp.parse(
                 raw.data,
                 raw.info,
-                self._data_fields
+                self._data_fields,
+                self.device
             )
 
         except RuntimeError as err:
@@ -578,7 +578,7 @@ class BiologicProgram( ABC ):
 
                 break
 
-            await asyncio.sleep( interval ) # wait
+            await asyncio.sleep( interval )  # wait
 
             # retrieve data
             active_channels = [ ch for ch, done in complete.items() if ( not done ) ]
